# -*- coding: utf-8 -*-
import json
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from django.views.generic.detail import View

from dashing.settings import dashing_settings


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        return HttpResponse(
            self.convert_context_to_json(context),
            content_type='application/json',
            **response_kwargs
        )

    def convert_context_to_json(self, context):
        "Convert the context dictionary into a JSON object"
        return json.dumps(context)


class Widget(JSONResponseMixin, View):
    permission_classes = dashing_settings.PERMISSION_CLASSES

    def check_permissions(self, request):
        """
        Check if the request should be permitted.
        Raises an appropriate exception if the request is not permitted.
        """
        permissions = [permission() for permission in self.permission_classes]
        for permission in permissions:
            if not permission.has_permission(request):
                raise PermissionDenied()

    def get(self, request, *args, **kwargs):
        self.check_permissions(request)
        context = json.dumps(self.get_context())
        return HttpResponse(context, content_type="application/json")

    def render_to_response(self, context, **response_kwargs):
        return self.render_to_json_response(context, **response_kwargs)


class NumberWidget(Widget):
    title = ''
    more_info = ''
    updated_at = ''
    detail = ''
    value = ''

    def get_title(self):
        return self.title

    def get_more_info(self):
        return self.more_info

    def get_updated_at(self):
        return self.updated_at

    def get_detail(self):
        return self.detail

    def get_value(self):
        return self.value

    def get_context(self):
        return {
            'title': self.get_title(),
            'moreInfo': self.get_more_info(),
            'updatedAt': self.get_updated_at(),
            'detail': self.get_detail(),
            'value': self.get_value(),
        }


class ListWidget(Widget):
    title = ''
    more_info = ''
    updated_at = ''
    data = []

    def get_title(self):
        return self.title

    def get_more_info(self):
        return self.more_info

    def get_updated_at(self):
        return self.updated_at

    def get_data(self):
        return self.data

    def get_context(self):
        return {
            'title': self.get_title(),
            'moreInfo': self.get_more_info(),
            'updatedAt': self.get_updated_at(),
            'data': self.get_data(),
        }


class GraphWidget(Widget):
    title = ''
    more_info = ''
    value = ''
    data = []
    properties = {}

    def get_title(self):
        return self.title

    def get_more_info(self):
        return self.more_info

    def get_value(self):
        return self.value

    def get_data(self):
        return self.data

    def get_properties(self):
        return self.properties

    def get_context(self):
        return {
            'title': self.get_title(),
            'moreInfo': self.get_more_info(),
            'value': self.get_value(),
            'data': self.get_data(),
            'properties': self.get_properties(),
        }


class KnobWidget(Widget):
    title = ''
    value = ''
    data = {}
    detail = ''
    more_info = ''
    updated_at = ''

    def get_title(self):
        return self.title

    def get_data(self):
        return self.data

    def get_detail(self):
        return self.detail

    def get_more_info(self):
        return self.more_info

    def get_value(self):
        return self.value

    def get_updated_at(self):
        return self.updated_at

    def get_context(self):
        return {
            'title': self.get_title(),
            'value': self.get_value(),
            'data': self.get_data(),
            'detail': self.get_detail(),
            'moreInfo': self.get_more_info(),
            'updatedAt': self.get_updated_at(),
        }


class MapWidget(Widget):
    theme = 'blue'
    zoom = 8
    double_click_zoom = True
    default_ui = False
    center = {}
    markers = []

    def get_theme(self):
        return self.theme

    def get_zoom(self):
        return self.zoom

    def get_double_click_zoom(self):
        return self.double_click_zoom

    def get_default_ui(self):
        return self.default_ui

    def get_center(self):
        return self.center

    def get_markers(self):
        return self.markers

    def get_context(self):
        return {
            'theme': self.get_theme(),
            'map': {
                'zoom': self.get_zoom(),
                'doubleClickZoom': self.get_double_click_zoom(),
                'defaultUI': self.get_default_ui(),
                'center': self.get_center(),
                'markers': self.get_markers()
            }
        }



import codecs
import json
import urllib.request
import urllib.error
import urllib.parse
from django import template
from django.conf import settings
from django.template import Node
from django.utils.safestring import mark_safe
from django.templatetags.static import static
from django.contrib.staticfiles.finders import find

from dashing.settings import dashing_settings


register = template.Library()

_resource = None


def remote_path(name):
    global _resource
    if not _resource:
        try:
            reader = codecs.getreader("utf-8")
            resource = json.load(reader(urllib.request.urlopen(dashing_settings.REPOSITORY)))
            _resource = resource
        except ValueError:
            raise ValueError('Repository format is incorrect')
    else:
        resource = _resource

    if 'widgets' in resource:
        repositories = {x['name']: x['repository'] for x in resource['widgets']}
    else:
        raise ValueError('No widgets specified in the repository')

    if name in repositories:
        return repositories[name]
    else:
        return None


def load(template, file_extension):
    widgets = dashing_settings.INSTALLED_WIDGETS
    output = ''
    for name in widgets:
        local_path = 'dashing/{}/{}/{}.{}'.format('widgets', name, name, file_extension)
        if find(local_path):
            output += template.format(static(local_path), name)
        else:
            path = remote_path(name)
            if path:
                path += '{}.{}'.format(name, file_extension)
                output += template.format(path, name)
    return mark_safe(output)


@register.simple_tag
def widget_styles():
    return load('<link rel="stylesheet" href="{}">\n', 'css')


@register.simple_tag
def widget_scripts():
    return load('<script type="text/javascript" src="{}"></script>\n', 'js')


@register.simple_tag
def widget_templates():
    return load('<link rel="resource" type="text/html" '
                'href="{}" data-widget="{}">\n', 'html')


@register.simple_tag
def widget_configs():
    widgets = dashing_settings.INSTALLED_WIDGETS
    output = ''
    for name in widgets:
        if name in dashing_settings.WIDGET_CONFIGS:
            for key, value in dashing_settings.WIDGET_CONFIGS[name].items():
                output += '<script type="text/javascript">var {} = "{}";</script>'.format(key, value)
    return mark_safe(output)


if "compressor" in settings.INSTALLED_APPS:
    @register.tag
    def compress(parser, token):
        """
        Shadows django-compressor's compress tag so it can be
        loaded from ``dashing_tags``, allowing us to provide
        a dummy version when django-compressor isn't installed.
        """
        from compressor.templatetags.compress import compress
        return compress(parser, token)
else:
    @register.tag
    def compress(parser, token):
        """
        Dummy tag for fallback when django-compressor isn't installed.
        """
        class TagNode(Node):
            def __init__(self, nodelist):
                self.nodelist = nodelist

            def render(self, context):
                return self.nodelist.render(context)
        nodelist = parser.parse(('endcompress',))
        parser.delete_first_token()
        return TagNode(nodelist)


@register.simple_tag
def moment_locales():
    locales = dashing_settings.LOCALES
    o = ''
    if len(locales) == 0:
        return o

    for locale in locales:
        src = 'dashing/libs/moment/locale/{}.js'.format(locale)
        o += ('<script type="text/javascript"'
              ' src="{}"></script>\n'.format(static(src)))
    o += ('<script type="text/javascript">'
          'moment.locale("{}");</script>\n'.format(locales[0]))
    return mark_safe(o)



# -*- encoding: utf-8 -*-
from django.views.generic import TemplateView
from django.core.exceptions import PermissionDenied

from dashing.settings import dashing_settings


class Dashboard(TemplateView):
    template_name = 'dashing/dashboard.html'
    permission_classes = dashing_settings.PERMISSION_CLASSES

    def check_permissions(self, request):
        """
        Check if the request should be permitted.
        Raises an appropriate exception if the request is not permitted.
        """
        permissions = [permission() for permission in self.permission_classes]
        for permission in permissions:
            if not permission.has_permission(request):
                raise PermissionDenied()

    def get(self, request, *args, **kwargs):
        self.check_permissions(request)
        return super(Dashboard, self).get(request, *args, **kwargs)



# -*- encoding: utf-8 -*-


class BasePermission(object):
    """
    A base class from which all permission classes should inherit.
    """

    def has_permission(self, request):
        """
        Return `True` if permission is granted, `False` otherwise.
        """
        return True


class AllowAny(BasePermission):
    """
    Allow any access.
    """

    def has_permission(self, request):
        return True


class IsAuthenticated(BasePermission):
    """
    Allows access only to authenticated users.
    """

    def has_permission(self, request):
        try:
            return request.user and request.user.is_authenticated()
        except TypeError:
            return request.user and request.user.is_authenticated


class IsAdminUser(BasePermission):
    """
    Allows access only to admin users.
    """

    def has_permission(self, request):
        return request.user and request.user.is_staff



from django.conf import settings
import importlib

USER_SETTINGS = getattr(settings, 'DASHING', None)

DEFAULTS = {
    'REPOSITORY': 'https://cdn.rawgit.com/talpor/django-dashing-channel/'
                  'master/repositories.json',
    'INSTALLED_WIDGETS': ('number', 'list', 'graph', 'clock', 'knob', 'map',),
    'PERMISSION_CLASSES':  (
        'dashing.permissions.AllowAny',
    ),
    'LOCALES': (),
    'WIDGET_CONFIGS': {},
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = ('PERMISSION_CLASSES',)


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        # Nod to tastypie's use of importlib.
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = importlib.import_module(module_path)
        return getattr(module, class_name)
    except ImportError as e:
        msg = ('Could not import \'{}\' '
               'for setting \'{}\'. {}: {}.').format(val,
                                                     setting_name,
                                                     e.__class__.__name__, e)
        raise ImportError(msg)


class Settings(object):
    """
    A settings object, that allows settings to be accessed as properties.
    For example:
        from dashing.settings import dashing_settings
        print dashing_settings.INSTALLED_WIDGETS
    Any setting with string import paths will be automatically resolved
    and return the class, rather than the string literal.
    """

    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        self.user_settings = user_settings or {}
        self.defaults = defaults or {}
        self.import_strings = import_strings or ()

    def __getattr__(self, attr):
        if attr not in self.defaults.keys():
            raise AttributeError("Invalid setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if val and attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val


dashing_settings = Settings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)



import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django3-dashing',
    version='0.5.1',
    packages=['dashing'],
    include_package_data=True,
    license='MIT License',
    description='A simple Django (v3 which requires Python 3) dashboard app to visualize interesting data '
                'about your project.',
    long_description=README,
    url='https://github.com/wickeyware/django-dashing/',
    author='Michael Wickey',
    author_email='wickeym@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Localization',
        'Topic :: Utilities',
    ],
    keywords=['django', 'python3', 'metrics', 'dashboard', 'dashing', 'metric',
              'widgets', 'data'],
)



from django.urls import path, re_path
from .views import Dashboard


class Router(object):
    def __init__(self):
        self.registry = []

    def register(self, widget, basename, **parameters):
        """ Register a widget, URL basename and any optional URL parameters.

        Parameters are passed as keyword arguments, i.e.
            >>> router.register(MyWidget, 'mywidget', my_parameter="[A-Z0-9]+")

        This would be the equivalent of manually adding the following
        to urlpatterns:
            >>> re_path(r"^widgets/mywidget/(P<my_parameter>[A-Z0-9]+)/?",
                                         MyWidget.as_view(), "widget_mywidget")

        """
        self.registry.append((widget, basename, parameters))

    def get_urls(self):
        urlpatterns = [
            path('', Dashboard.as_view(), name='dashboard'),
        ]

        for widget, basename, parameters in self.registry:
            urlpatterns += [
                re_path(r'/'.join((
                    r'^widgets/{}'.format(basename),
                    r'/'.join((r'(?P<{}>{})'.format(parameter, regex)
                               for parameter, regex in parameters.items())),
                )),
                    widget.as_view(),
                    name='widget_{}'.format(basename)),
            ]

        return urlpatterns

    @property
    def urls(self):
        return self.get_urls()


router = Router()
